using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Configuration.Provider;
using System.IO;
using System.Text;
using System.Web.Configuration;

namespace WebTools.Configuration.Provider
{
	/// <summary>
	/// Provides static methods for retrieving configuration values.
	/// </summary>
	public static class ProviderUtil
	{
		/// <summary>
		/// Gets the value of the applicationName attribute and removes the entry from the config collection.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <returns>The value of the applicationName attribute.</returns>
		public static string GetAppName(NameValueCollection config)
		{
			string appName = config["applicationName"];
			if (String.IsNullOrEmpty(appName))
			{
				appName = "WebTools";
			}
			config.Remove("applicationName");
			return appName;
		}

		/// <summary>
		/// Gets the value of the connectionStringName attribute and removes the entry from the config collection.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <returns>The value of the connectionStringName attribute.</returns>
		public static string GetConnectionString(NameValueCollection config)
		{
			string connStr = WebConfigurationManager.ConnectionStrings[config["connectionStringName"]].ConnectionString;
			config.Remove("connectionStringName");
			return connStr;
		}

		/// <summary>
		/// Gets the string value of the specified attribute and removes the entry from the config collection.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <param name="attributeName">The name of attribute to find within config.</param>
		/// <returns>The string value of the specified attribute.</returns>
		public static string GetAndRemoveStringAttribute(NameValueCollection config, string attributeName)
		{
			string val = config.Get(attributeName);
			if (val != null)
			{
				config.Remove(attributeName);
				return val;
			}
			return String.Empty;
		}

		/// <summary>
		/// Gets the Int32 value of the specified attribute and removes the entry from the config collection.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <param name="attributeName">The name of attribute to find within config.</param>
		/// <returns>The Int32 value of the specified attribute.</returns>
		public static int GetAndRemoveInt32Attribute(NameValueCollection config, string attributeName)
		{
			string val = config.Get(attributeName);
			if (!String.IsNullOrEmpty(val))
			{
				int result;
				if (Int32.TryParse(val, out result))
				{
					config.Remove(attributeName);
					return result;
				}
			}
			return 0;
		}

		/// <summary>
		/// Gets the Boolean value of the specified attribute and removes the entry from the config collection.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <param name="attributeName">The name of attribute to find within config.</param>
		/// <returns>The Boolean value of the specified attribute.</returns>
		public static bool GetAndRemoveBooleanAttribute(NameValueCollection config, string attributeName)
		{
			string val = config.Get(attributeName);
			if (!String.IsNullOrEmpty(val))
			{
				bool result;
				if (Boolean.TryParse(val, out result))
				{
					config.Remove(attributeName);
					return result;
				}
			}
			return false;
		}

		/// <summary>
		/// Checks the config collection to determine if the collection is emtpy, indicating that all entries have been handled.
		/// </summary>
		/// <param name="config">The collection of configuration key/value pairs.</param>
		/// <exception cref="ConfigurationErrorsException">The config collection contains unrecognized attributes.</exception>
		public static void CheckUnrecognizedAttributes(NameValueCollection config)
		{
			if (config.Count > 0)
			{
				string key = config.GetKey(0);
				if (!String.IsNullOrEmpty(key))
				{
					throw new ConfigurationErrorsException(String.Format("The attribute '{0}' is not a recognized attribute for this provider.", key));
				}
			}
		}
	}
}
